# Generated by cloudcoil-model-codegen v0.3.0
# DO NOT EDIT

from __future__ import annotations

from typing import (
    Annotated,
    Callable,
    List,
    Literal,
    Optional,
    Type,
    overload,
)

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil.pydantic import (
    BaseModel,
    BaseModelBuilder,
    BuilderContextBase,
    GenericListBuilder,
    ListBuilderContext,
    Never,
    Self,
)
from cloudcoil.resources import Resource, ResourceList


class ServerStorageVersion(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["ServerStorageVersion"]:
            return ServerStorageVersion

        def build(self) -> "ServerStorageVersion":
            return ServerStorageVersion(**self._attrs)

        def api_server_id(self, value: Optional[str], /) -> Self:
            """
            The ID of the reporting API server.
            """
            return self._set("api_server_id", value)

        def decodable_versions(self, value: Optional[List[str]], /) -> Self:
            """
            The API server can decode objects encoded in these versions. The encodingVersion must be included in the decodableVersions.
            """
            return self._set("decodable_versions", value)

        def encoding_version(self, value: Optional[str], /) -> Self:
            """
            The API server encodes the object to this version when persisting it in the backend (e.g., etcd).
            """
            return self._set("encoding_version", value)

        def served_versions(self, value: Optional[List[str]], /) -> Self:
            """
            The API server can serve these versions. DecodableVersions must include all ServedVersions.
            """
            return self._set("served_versions", value)

    class BuilderContext(BuilderContextBase["ServerStorageVersion.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = ServerStorageVersion.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for ServerStorageVersion."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["ServerStorageVersion", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use ServerStorageVersion.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    api_server_id: Annotated[Optional[str], Field(alias="apiServerID")] = None
    """
    The ID of the reporting API server.
    """
    decodable_versions: Annotated[Optional[List[str]], Field(alias="decodableVersions")] = None
    """
    The API server can decode objects encoded in these versions. The encodingVersion must be included in the decodableVersions.
    """
    encoding_version: Annotated[Optional[str], Field(alias="encodingVersion")] = None
    """
    The API server encodes the object to this version when persisting it in the backend (e.g., etcd).
    """
    served_versions: Annotated[Optional[List[str]], Field(alias="servedVersions")] = None
    """
    The API server can serve these versions. DecodableVersions must include all ServedVersions.
    """


class StorageVersionSpec(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["StorageVersionSpec"]:
            return StorageVersionSpec

        def build(self) -> "StorageVersionSpec":
            return StorageVersionSpec(**self._attrs)

    class BuilderContext(BuilderContextBase["StorageVersionSpec.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = StorageVersionSpec.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for StorageVersionSpec."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["StorageVersionSpec", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use StorageVersionSpec.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    pass


class StorageVersionCondition(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["StorageVersionCondition"]:
            return StorageVersionCondition

        def build(self) -> "StorageVersionCondition":
            return StorageVersionCondition(**self._attrs)

        @overload
        def last_transition_time(
            self, value_or_callback: Optional[apimachinery.Time], /
        ) -> "StorageVersionCondition.Builder": ...

        @overload
        def last_transition_time(
            self,
            value_or_callback: Callable[
                [apimachinery.Time.Builder],
                apimachinery.Time.Builder | apimachinery.Time,
            ],
            /,
        ) -> "StorageVersionCondition.Builder": ...

        @overload
        def last_transition_time(
            self, value_or_callback: Never = ...
        ) -> "apimachinery.Time.BuilderContext": ...

        def last_transition_time(self, value_or_callback=None, /):
            """
            Last time the condition transitioned from one status to another.
            """
            if self._in_context and value_or_callback is None:
                context = apimachinery.Time.BuilderContext()
                context._parent_builder = self
                context._field_name = "last_transition_time"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(apimachinery.Time.builder())
                if isinstance(output, apimachinery.Time.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("last_transition_time", value)

        def message(self, value: Optional[str], /) -> Self:
            """
            A human readable message indicating details about the transition.
            """
            return self._set("message", value)

        def observed_generation(self, value: Optional[int], /) -> Self:
            """
            If set, this represents the .metadata.generation that the condition was set based upon.
            """
            return self._set("observed_generation", value)

        def reason(self, value: str, /) -> Self:
            """
            The reason for the condition's last transition.
            """
            return self._set("reason", value)

        def status(self, value: str, /) -> Self:
            """
            Status of the condition, one of True, False, Unknown.
            """
            return self._set("status", value)

        def type(self, value: str, /) -> Self:
            """
            Type of the condition.
            """
            return self._set("type", value)

    class BuilderContext(BuilderContextBase["StorageVersionCondition.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = StorageVersionCondition.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for StorageVersionCondition."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["StorageVersionCondition", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use StorageVersionCondition.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    last_transition_time: Annotated[
        Optional[apimachinery.Time], Field(alias="lastTransitionTime")
    ] = None
    """
    Last time the condition transitioned from one status to another.
    """
    message: Optional[str] = None
    """
    A human readable message indicating details about the transition.
    """
    observed_generation: Annotated[Optional[int], Field(alias="observedGeneration")] = None
    """
    If set, this represents the .metadata.generation that the condition was set based upon.
    """
    reason: str
    """
    The reason for the condition's last transition.
    """
    status: str
    """
    Status of the condition, one of True, False, Unknown.
    """
    type: str
    """
    Type of the condition.
    """


class StorageVersionStatus(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["StorageVersionStatus"]:
            return StorageVersionStatus

        def build(self) -> "StorageVersionStatus":
            return StorageVersionStatus(**self._attrs)

        def common_encoding_version(self, value: Optional[str], /) -> Self:
            """
            If all API server instances agree on the same encoding storage version, then this field is set to that version. Otherwise this field is left empty. API servers should finish updating its storageVersionStatus entry before serving write operations, so that this field will be in sync with the reality.
            """
            return self._set("common_encoding_version", value)

        @overload
        def conditions(
            self, value_or_callback: List[StorageVersionCondition], /
        ) -> "StorageVersionStatus.Builder": ...

        @overload
        def conditions(
            self,
            value_or_callback: Callable[
                [GenericListBuilder[StorageVersionCondition, StorageVersionCondition.Builder]],
                GenericListBuilder[StorageVersionCondition, StorageVersionCondition.Builder]
                | List[StorageVersionCondition],
            ],
            /,
        ) -> "StorageVersionStatus.Builder": ...

        @overload
        def conditions(
            self, value_or_callback: Never = ...
        ) -> ListBuilderContext[StorageVersionCondition.Builder]: ...

        def conditions(self, value_or_callback=None, /):
            """
            The latest available observations of the storageVersion's state.
            """
            if self._in_context and value_or_callback is None:
                context = ListBuilderContext[StorageVersionCondition.Builder]()
                context._parent_builder = self
                context._field_name = "conditions"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(StorageVersionCondition.list_builder())
                if isinstance(output, GenericListBuilder):
                    value = output.build()
                else:
                    value = output
            return self._set("conditions", value)

        @overload
        def storage_versions(
            self, value_or_callback: List[ServerStorageVersion], /
        ) -> "StorageVersionStatus.Builder": ...

        @overload
        def storage_versions(
            self,
            value_or_callback: Callable[
                [GenericListBuilder[ServerStorageVersion, ServerStorageVersion.Builder]],
                GenericListBuilder[ServerStorageVersion, ServerStorageVersion.Builder]
                | List[ServerStorageVersion],
            ],
            /,
        ) -> "StorageVersionStatus.Builder": ...

        @overload
        def storage_versions(
            self, value_or_callback: Never = ...
        ) -> ListBuilderContext[ServerStorageVersion.Builder]: ...

        def storage_versions(self, value_or_callback=None, /):
            """
            The reported versions per API server instance.
            """
            if self._in_context and value_or_callback is None:
                context = ListBuilderContext[ServerStorageVersion.Builder]()
                context._parent_builder = self
                context._field_name = "storage_versions"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(ServerStorageVersion.list_builder())
                if isinstance(output, GenericListBuilder):
                    value = output.build()
                else:
                    value = output
            return self._set("storage_versions", value)

    class BuilderContext(BuilderContextBase["StorageVersionStatus.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = StorageVersionStatus.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for StorageVersionStatus."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["StorageVersionStatus", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use StorageVersionStatus.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    common_encoding_version: Annotated[Optional[str], Field(alias="commonEncodingVersion")] = None
    """
    If all API server instances agree on the same encoding storage version, then this field is set to that version. Otherwise this field is left empty. API servers should finish updating its storageVersionStatus entry before serving write operations, so that this field will be in sync with the reality.
    """
    conditions: Optional[List[StorageVersionCondition]] = None
    """
    The latest available observations of the storageVersion's state.
    """
    storage_versions: Annotated[
        Optional[List[ServerStorageVersion]], Field(alias="storageVersions")
    ] = None
    """
    The reported versions per API server instance.
    """


class StorageVersion(Resource):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["StorageVersion"]:
            return StorageVersion

        def build(self) -> "StorageVersion":
            return StorageVersion(**self._attrs)

        def api_version(
            self, value: Optional[Literal["internal.apiserver.k8s.io/v1alpha1"]], /
        ) -> Self:
            """
            APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
            """
            return self._set("api_version", value)

        def kind(self, value: Optional[Literal["StorageVersion"]], /) -> Self:
            """
            Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
            """
            return self._set("kind", value)

        @overload
        def metadata(
            self, value_or_callback: Optional[apimachinery.ObjectMeta], /
        ) -> "StorageVersion.Builder": ...

        @overload
        def metadata(
            self,
            value_or_callback: Callable[
                [apimachinery.ObjectMeta.Builder],
                apimachinery.ObjectMeta.Builder | apimachinery.ObjectMeta,
            ],
            /,
        ) -> "StorageVersion.Builder": ...

        @overload
        def metadata(
            self, value_or_callback: Never = ...
        ) -> "apimachinery.ObjectMeta.BuilderContext": ...

        def metadata(self, value_or_callback=None, /):
            """
            The name is <group>.<resource>.
            """
            if self._in_context and value_or_callback is None:
                context = apimachinery.ObjectMeta.BuilderContext()
                context._parent_builder = self
                context._field_name = "metadata"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(apimachinery.ObjectMeta.builder())
                if isinstance(output, apimachinery.ObjectMeta.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("metadata", value)

        @overload
        def spec(self, value_or_callback: StorageVersionSpec, /) -> "StorageVersion.Builder": ...

        @overload
        def spec(
            self,
            value_or_callback: Callable[
                [StorageVersionSpec.Builder],
                StorageVersionSpec.Builder | StorageVersionSpec,
            ],
            /,
        ) -> "StorageVersion.Builder": ...

        @overload
        def spec(self, value_or_callback: Never = ...) -> "StorageVersionSpec.BuilderContext": ...

        def spec(self, value_or_callback=None, /):
            """
            Spec is an empty spec. It is here to comply with Kubernetes API style.
            """
            if self._in_context and value_or_callback is None:
                context = StorageVersionSpec.BuilderContext()
                context._parent_builder = self
                context._field_name = "spec"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(StorageVersionSpec.builder())
                if isinstance(output, StorageVersionSpec.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("spec", value)

        @overload
        def status(
            self, value_or_callback: StorageVersionStatus, /
        ) -> "StorageVersion.Builder": ...

        @overload
        def status(
            self,
            value_or_callback: Callable[
                [StorageVersionStatus.Builder],
                StorageVersionStatus.Builder | StorageVersionStatus,
            ],
            /,
        ) -> "StorageVersion.Builder": ...

        @overload
        def status(
            self, value_or_callback: Never = ...
        ) -> "StorageVersionStatus.BuilderContext": ...

        def status(self, value_or_callback=None, /):
            """
            API server instances report the version they can decode and the version they encode objects to when persisting objects in the backend.
            """
            if self._in_context and value_or_callback is None:
                context = StorageVersionStatus.BuilderContext()
                context._parent_builder = self
                context._field_name = "status"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(StorageVersionStatus.builder())
                if isinstance(output, StorageVersionStatus.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("status", value)

    class BuilderContext(BuilderContextBase["StorageVersion.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = StorageVersion.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for StorageVersion."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["StorageVersion", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use StorageVersion.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    api_version: Annotated[
        Optional[Literal["internal.apiserver.k8s.io/v1alpha1"]],
        Field(alias="apiVersion"),
    ] = "internal.apiserver.k8s.io/v1alpha1"
    """
    APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
    """
    kind: Optional[Literal["StorageVersion"]] = "StorageVersion"
    """
    Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
    """
    metadata: Optional[apimachinery.ObjectMeta] = None
    """
    The name is <group>.<resource>.
    """
    spec: StorageVersionSpec
    """
    Spec is an empty spec. It is here to comply with Kubernetes API style.
    """
    status: StorageVersionStatus
    """
    API server instances report the version they can decode and the version they encode objects to when persisting objects in the backend.
    """


StorageVersionList = ResourceList["StorageVersion"]



# Generated by cloudcoil-model-codegen v0.3.0
# DO NOT EDIT

from __future__ import annotations

from typing import (
    Annotated,
    Callable,
    List,
    Literal,
    Optional,
    Type,
    overload,
)

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil.pydantic import (
    BaseModel,
    BaseModelBuilder,
    BuilderContextBase,
    GenericListBuilder,
    ListBuilderContext,
    Never,
    Self,
)
from cloudcoil.resources import Resource, ResourceList

from ..core import v1


class PodSchedulingContextSpec(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["PodSchedulingContextSpec"]:
            return PodSchedulingContextSpec

        def build(self) -> "PodSchedulingContextSpec":
            return PodSchedulingContextSpec(**self._attrs)

        def potential_nodes(self, value: Optional[List[str]], /) -> Self:
            """
            PotentialNodes lists nodes where the Pod might be able to run.

            The size of this field is limited to 128. This is large enough for many clusters. Larger clusters may need more attempts to find a node that suits all pending resources. This may get increased in the future, but not reduced.
            """
            return self._set("potential_nodes", value)

        def selected_node(self, value: Optional[str], /) -> Self:
            """
            SelectedNode is the node for which allocation of ResourceClaims that are referenced by the Pod and that use "WaitForFirstConsumer" allocation is to be attempted.
            """
            return self._set("selected_node", value)

    class BuilderContext(BuilderContextBase["PodSchedulingContextSpec.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = PodSchedulingContextSpec.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for PodSchedulingContextSpec."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["PodSchedulingContextSpec", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use PodSchedulingContextSpec.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    potential_nodes: Annotated[Optional[List[str]], Field(alias="potentialNodes")] = None
    """
    PotentialNodes lists nodes where the Pod might be able to run.

    The size of this field is limited to 128. This is large enough for many clusters. Larger clusters may need more attempts to find a node that suits all pending resources. This may get increased in the future, but not reduced.
    """
    selected_node: Annotated[Optional[str], Field(alias="selectedNode")] = None
    """
    SelectedNode is the node for which allocation of ResourceClaims that are referenced by the Pod and that use "WaitForFirstConsumer" allocation is to be attempted.
    """


class ResourceClaimConsumerReference(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["ResourceClaimConsumerReference"]:
            return ResourceClaimConsumerReference

        def build(self) -> "ResourceClaimConsumerReference":
            return ResourceClaimConsumerReference(**self._attrs)

        def api_group(self, value: Optional[str], /) -> Self:
            """
            APIGroup is the group for the resource being referenced. It is empty for the core API. This matches the group in the APIVersion that is used when creating the resources.
            """
            return self._set("api_group", value)

        def name(self, value: str, /) -> Self:
            """
            Name is the name of resource being referenced.
            """
            return self._set("name", value)

        def resource(self, value: str, /) -> Self:
            """
            Resource is the type of resource being referenced, for example "pods".
            """
            return self._set("resource", value)

        def uid(self, value: str, /) -> Self:
            """
            UID identifies exactly one incarnation of the resource.
            """
            return self._set("uid", value)

    class BuilderContext(BuilderContextBase["ResourceClaimConsumerReference.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = ResourceClaimConsumerReference.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for ResourceClaimConsumerReference."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["ResourceClaimConsumerReference", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use ResourceClaimConsumerReference.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    api_group: Annotated[Optional[str], Field(alias="apiGroup")] = None
    """
    APIGroup is the group for the resource being referenced. It is empty for the core API. This matches the group in the APIVersion that is used when creating the resources.
    """
    name: str
    """
    Name is the name of resource being referenced.
    """
    resource: str
    """
    Resource is the type of resource being referenced, for example "pods".
    """
    uid: str
    """
    UID identifies exactly one incarnation of the resource.
    """


class ResourceClaimParametersReference(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["ResourceClaimParametersReference"]:
            return ResourceClaimParametersReference

        def build(self) -> "ResourceClaimParametersReference":
            return ResourceClaimParametersReference(**self._attrs)

        def api_group(self, value: Optional[str], /) -> Self:
            """
            APIGroup is the group for the resource being referenced. It is empty for the core API. This matches the group in the APIVersion that is used when creating the resources.
            """
            return self._set("api_group", value)

        def kind(self, value: str, /) -> Self:
            """
            Kind is the type of resource being referenced. This is the same value as in the parameter object's metadata, for example "ConfigMap".
            """
            return self._set("kind", value)

        def name(self, value: str, /) -> Self:
            """
            Name is the name of resource being referenced.
            """
            return self._set("name", value)

    class BuilderContext(BuilderContextBase["ResourceClaimParametersReference.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = ResourceClaimParametersReference.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for ResourceClaimParametersReference."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["ResourceClaimParametersReference", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use ResourceClaimParametersReference.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    api_group: Annotated[Optional[str], Field(alias="apiGroup")] = None
    """
    APIGroup is the group for the resource being referenced. It is empty for the core API. This matches the group in the APIVersion that is used when creating the resources.
    """
    kind: str
    """
    Kind is the type of resource being referenced. This is the same value as in the parameter object's metadata, for example "ConfigMap".
    """
    name: str
    """
    Name is the name of resource being referenced.
    """


class ResourceClaimSchedulingStatus(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["ResourceClaimSchedulingStatus"]:
            return ResourceClaimSchedulingStatus

        def build(self) -> "ResourceClaimSchedulingStatus":
            return ResourceClaimSchedulingStatus(**self._attrs)

        def name(self, value: Optional[str], /) -> Self:
            """
            Name matches the pod.spec.resourceClaims[*].Name field.
            """
            return self._set("name", value)

        def unsuitable_nodes(self, value: Optional[List[str]], /) -> Self:
            """
            UnsuitableNodes lists nodes that the ResourceClaim cannot be allocated for.

            The size of this field is limited to 128, the same as for PodSchedulingSpec.PotentialNodes. This may get increased in the future, but not reduced.
            """
            return self._set("unsuitable_nodes", value)

    class BuilderContext(BuilderContextBase["ResourceClaimSchedulingStatus.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = ResourceClaimSchedulingStatus.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for ResourceClaimSchedulingStatus."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["ResourceClaimSchedulingStatus", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use ResourceClaimSchedulingStatus.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    name: Optional[str] = None
    """
    Name matches the pod.spec.resourceClaims[*].Name field.
    """
    unsuitable_nodes: Annotated[Optional[List[str]], Field(alias="unsuitableNodes")] = None
    """
    UnsuitableNodes lists nodes that the ResourceClaim cannot be allocated for.

    The size of this field is limited to 128, the same as for PodSchedulingSpec.PotentialNodes. This may get increased in the future, but not reduced.
    """


class ResourceClaimSpec(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["ResourceClaimSpec"]:
            return ResourceClaimSpec

        def build(self) -> "ResourceClaimSpec":
            return ResourceClaimSpec(**self._attrs)

        def allocation_mode(self, value: Optional[str], /) -> Self:
            """
            Allocation can start immediately or when a Pod wants to use the resource. "WaitForFirstConsumer" is the default.
            """
            return self._set("allocation_mode", value)

        @overload
        def parameters_ref(
            self, value_or_callback: Optional[ResourceClaimParametersReference], /
        ) -> "ResourceClaimSpec.Builder": ...

        @overload
        def parameters_ref(
            self,
            value_or_callback: Callable[
                [ResourceClaimParametersReference.Builder],
                ResourceClaimParametersReference.Builder | ResourceClaimParametersReference,
            ],
            /,
        ) -> "ResourceClaimSpec.Builder": ...

        @overload
        def parameters_ref(
            self, value_or_callback: Never = ...
        ) -> "ResourceClaimParametersReference.BuilderContext": ...

        def parameters_ref(self, value_or_callback=None, /):
            """
            ParametersRef references a separate object with arbitrary parameters that will be used by the driver when allocating a resource for the claim.

            The object must be in the same namespace as the ResourceClaim.
            """
            if self._in_context and value_or_callback is None:
                context = ResourceClaimParametersReference.BuilderContext()
                context._parent_builder = self
                context._field_name = "parameters_ref"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(ResourceClaimParametersReference.builder())
                if isinstance(output, ResourceClaimParametersReference.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("parameters_ref", value)

        def resource_class_name(self, value: str, /) -> Self:
            """
            ResourceClassName references the driver and additional parameters via the name of a ResourceClass that was created as part of the driver deployment.
            """
            return self._set("resource_class_name", value)

    class BuilderContext(BuilderContextBase["ResourceClaimSpec.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = ResourceClaimSpec.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for ResourceClaimSpec."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["ResourceClaimSpec", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use ResourceClaimSpec.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    allocation_mode: Annotated[Optional[str], Field(alias="allocationMode")] = None
    """
    Allocation can start immediately or when a Pod wants to use the resource. "WaitForFirstConsumer" is the default.
    """
    parameters_ref: Annotated[
        Optional[ResourceClaimParametersReference], Field(alias="parametersRef")
    ] = None
    """
    ParametersRef references a separate object with arbitrary parameters that will be used by the driver when allocating a resource for the claim.

    The object must be in the same namespace as the ResourceClaim.
    """
    resource_class_name: Annotated[str, Field(alias="resourceClassName")]
    """
    ResourceClassName references the driver and additional parameters via the name of a ResourceClass that was created as part of the driver deployment.
    """


class ResourceClassParametersReference(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["ResourceClassParametersReference"]:
            return ResourceClassParametersReference

        def build(self) -> "ResourceClassParametersReference":
            return ResourceClassParametersReference(**self._attrs)

        def api_group(self, value: Optional[str], /) -> Self:
            """
            APIGroup is the group for the resource being referenced. It is empty for the core API. This matches the group in the APIVersion that is used when creating the resources.
            """
            return self._set("api_group", value)

        def kind(self, value: str, /) -> Self:
            """
            Kind is the type of resource being referenced. This is the same value as in the parameter object's metadata.
            """
            return self._set("kind", value)

        def name(self, value: str, /) -> Self:
            """
            Name is the name of resource being referenced.
            """
            return self._set("name", value)

        def namespace(self, value: Optional[str], /) -> Self:
            """
            Namespace that contains the referenced resource. Must be empty for cluster-scoped resources and non-empty for namespaced resources.
            """
            return self._set("namespace", value)

    class BuilderContext(BuilderContextBase["ResourceClassParametersReference.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = ResourceClassParametersReference.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for ResourceClassParametersReference."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["ResourceClassParametersReference", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use ResourceClassParametersReference.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    api_group: Annotated[Optional[str], Field(alias="apiGroup")] = None
    """
    APIGroup is the group for the resource being referenced. It is empty for the core API. This matches the group in the APIVersion that is used when creating the resources.
    """
    kind: str
    """
    Kind is the type of resource being referenced. This is the same value as in the parameter object's metadata.
    """
    name: str
    """
    Name is the name of resource being referenced.
    """
    namespace: Optional[str] = None
    """
    Namespace that contains the referenced resource. Must be empty for cluster-scoped resources and non-empty for namespaced resources.
    """


class ResourceHandle(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["ResourceHandle"]:
            return ResourceHandle

        def build(self) -> "ResourceHandle":
            return ResourceHandle(**self._attrs)

        def data(self, value: Optional[str], /) -> Self:
            """
            Data contains the opaque data associated with this ResourceHandle. It is set by the controller component of the resource driver whose name matches the DriverName set in the ResourceClaimStatus this ResourceHandle is embedded in. It is set at allocation time and is intended for processing by the kubelet plugin whose name matches the DriverName set in this ResourceHandle.

            The maximum size of this field is 16KiB. This may get increased in the future, but not reduced.
            """
            return self._set("data", value)

        def driver_name(self, value: Optional[str], /) -> Self:
            """
            DriverName specifies the name of the resource driver whose kubelet plugin should be invoked to process this ResourceHandle's data once it lands on a node. This may differ from the DriverName set in ResourceClaimStatus this ResourceHandle is embedded in.
            """
            return self._set("driver_name", value)

    class BuilderContext(BuilderContextBase["ResourceHandle.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = ResourceHandle.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for ResourceHandle."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["ResourceHandle", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use ResourceHandle.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    data: Optional[str] = None
    """
    Data contains the opaque data associated with this ResourceHandle. It is set by the controller component of the resource driver whose name matches the DriverName set in the ResourceClaimStatus this ResourceHandle is embedded in. It is set at allocation time and is intended for processing by the kubelet plugin whose name matches the DriverName set in this ResourceHandle.

    The maximum size of this field is 16KiB. This may get increased in the future, but not reduced.
    """
    driver_name: Annotated[Optional[str], Field(alias="driverName")] = None
    """
    DriverName specifies the name of the resource driver whose kubelet plugin should be invoked to process this ResourceHandle's data once it lands on a node. This may differ from the DriverName set in ResourceClaimStatus this ResourceHandle is embedded in.
    """


class AllocationResult(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["AllocationResult"]:
            return AllocationResult

        def build(self) -> "AllocationResult":
            return AllocationResult(**self._attrs)

        @overload
        def available_on_nodes(
            self, value_or_callback: Optional[v1.NodeSelector], /
        ) -> "AllocationResult.Builder": ...

        @overload
        def available_on_nodes(
            self,
            value_or_callback: Callable[
                [v1.NodeSelector.Builder], v1.NodeSelector.Builder | v1.NodeSelector
            ],
            /,
        ) -> "AllocationResult.Builder": ...

        @overload
        def available_on_nodes(
            self, value_or_callback: Never = ...
        ) -> "v1.NodeSelector.BuilderContext": ...

        def available_on_nodes(self, value_or_callback=None, /):
            """
            This field will get set by the resource driver after it has allocated the resource to inform the scheduler where it can schedule Pods using the ResourceClaim.

            Setting this field is optional. If null, the resource is available everywhere.
            """
            if self._in_context and value_or_callback is None:
                context = v1.NodeSelector.BuilderContext()
                context._parent_builder = self
                context._field_name = "available_on_nodes"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(v1.NodeSelector.builder())
                if isinstance(output, v1.NodeSelector.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("available_on_nodes", value)

        @overload
        def resource_handles(
            self, value_or_callback: List[ResourceHandle], /
        ) -> "AllocationResult.Builder": ...

        @overload
        def resource_handles(
            self,
            value_or_callback: Callable[
                [GenericListBuilder[ResourceHandle, ResourceHandle.Builder]],
                GenericListBuilder[ResourceHandle, ResourceHandle.Builder] | List[ResourceHandle],
            ],
            /,
        ) -> "AllocationResult.Builder": ...

        @overload
        def resource_handles(
            self, value_or_callback: Never = ...
        ) -> ListBuilderContext[ResourceHandle.Builder]: ...

        def resource_handles(self, value_or_callback=None, /):
            """
            ResourceHandles contain the state associated with an allocation that should be maintained throughout the lifetime of a claim. Each ResourceHandle contains data that should be passed to a specific kubelet plugin once it lands on a node. This data is returned by the driver after a successful allocation and is opaque to Kubernetes. Driver documentation may explain to users how to interpret this data if needed.

            Setting this field is optional. It has a maximum size of 32 entries. If null (or empty), it is assumed this allocation will be processed by a single kubelet plugin with no ResourceHandle data attached. The name of the kubelet plugin invoked will match the DriverName set in the ResourceClaimStatus this AllocationResult is embedded in.
            """
            if self._in_context and value_or_callback is None:
                context = ListBuilderContext[ResourceHandle.Builder]()
                context._parent_builder = self
                context._field_name = "resource_handles"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(ResourceHandle.list_builder())
                if isinstance(output, GenericListBuilder):
                    value = output.build()
                else:
                    value = output
            return self._set("resource_handles", value)

        def shareable(self, value: Optional[bool], /) -> Self:
            """
            Shareable determines whether the resource supports more than one consumer at a time.
            """
            return self._set("shareable", value)

    class BuilderContext(BuilderContextBase["AllocationResult.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = AllocationResult.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for AllocationResult."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["AllocationResult", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use AllocationResult.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    available_on_nodes: Annotated[Optional[v1.NodeSelector], Field(alias="availableOnNodes")] = None
    """
    This field will get set by the resource driver after it has allocated the resource to inform the scheduler where it can schedule Pods using the ResourceClaim.

    Setting this field is optional. If null, the resource is available everywhere.
    """
    resource_handles: Annotated[Optional[List[ResourceHandle]], Field(alias="resourceHandles")] = (
        None
    )
    """
    ResourceHandles contain the state associated with an allocation that should be maintained throughout the lifetime of a claim. Each ResourceHandle contains data that should be passed to a specific kubelet plugin once it lands on a node. This data is returned by the driver after a successful allocation and is opaque to Kubernetes. Driver documentation may explain to users how to interpret this data if needed.

    Setting this field is optional. It has a maximum size of 32 entries. If null (or empty), it is assumed this allocation will be processed by a single kubelet plugin with no ResourceHandle data attached. The name of the kubelet plugin invoked will match the DriverName set in the ResourceClaimStatus this AllocationResult is embedded in.
    """
    shareable: Optional[bool] = None
    """
    Shareable determines whether the resource supports more than one consumer at a time.
    """


class PodSchedulingContextStatus(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["PodSchedulingContextStatus"]:
            return PodSchedulingContextStatus

        def build(self) -> "PodSchedulingContextStatus":
            return PodSchedulingContextStatus(**self._attrs)

        @overload
        def resource_claims(
            self, value_or_callback: List[ResourceClaimSchedulingStatus], /
        ) -> "PodSchedulingContextStatus.Builder": ...

        @overload
        def resource_claims(
            self,
            value_or_callback: Callable[
                [
                    GenericListBuilder[
                        ResourceClaimSchedulingStatus,
                        ResourceClaimSchedulingStatus.Builder,
                    ]
                ],
                GenericListBuilder[
                    ResourceClaimSchedulingStatus, ResourceClaimSchedulingStatus.Builder
                ]
                | List[ResourceClaimSchedulingStatus],
            ],
            /,
        ) -> "PodSchedulingContextStatus.Builder": ...

        @overload
        def resource_claims(
            self, value_or_callback: Never = ...
        ) -> ListBuilderContext[ResourceClaimSchedulingStatus.Builder]: ...

        def resource_claims(self, value_or_callback=None, /):
            """
            ResourceClaims describes resource availability for each pod.spec.resourceClaim entry where the corresponding ResourceClaim uses "WaitForFirstConsumer" allocation mode.
            """
            if self._in_context and value_or_callback is None:
                context = ListBuilderContext[ResourceClaimSchedulingStatus.Builder]()
                context._parent_builder = self
                context._field_name = "resource_claims"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(ResourceClaimSchedulingStatus.list_builder())
                if isinstance(output, GenericListBuilder):
                    value = output.build()
                else:
                    value = output
            return self._set("resource_claims", value)

    class BuilderContext(BuilderContextBase["PodSchedulingContextStatus.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = PodSchedulingContextStatus.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for PodSchedulingContextStatus."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["PodSchedulingContextStatus", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use PodSchedulingContextStatus.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    resource_claims: Annotated[
        Optional[List[ResourceClaimSchedulingStatus]], Field(alias="resourceClaims")
    ] = None
    """
    ResourceClaims describes resource availability for each pod.spec.resourceClaim entry where the corresponding ResourceClaim uses "WaitForFirstConsumer" allocation mode.
    """


class ResourceClaimStatus(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["ResourceClaimStatus"]:
            return ResourceClaimStatus

        def build(self) -> "ResourceClaimStatus":
            return ResourceClaimStatus(**self._attrs)

        @overload
        def allocation(
            self, value_or_callback: Optional[AllocationResult], /
        ) -> "ResourceClaimStatus.Builder": ...

        @overload
        def allocation(
            self,
            value_or_callback: Callable[
                [AllocationResult.Builder], AllocationResult.Builder | AllocationResult
            ],
            /,
        ) -> "ResourceClaimStatus.Builder": ...

        @overload
        def allocation(
            self, value_or_callback: Never = ...
        ) -> "AllocationResult.BuilderContext": ...

        def allocation(self, value_or_callback=None, /):
            """
            Allocation is set by the resource driver once a resource or set of resources has been allocated successfully. If this is not specified, the resources have not been allocated yet.
            """
            if self._in_context and value_or_callback is None:
                context = AllocationResult.BuilderContext()
                context._parent_builder = self
                context._field_name = "allocation"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(AllocationResult.builder())
                if isinstance(output, AllocationResult.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("allocation", value)

        def deallocation_requested(self, value: Optional[bool], /) -> Self:
            """
            DeallocationRequested indicates that a ResourceClaim is to be deallocated.

            The driver then must deallocate this claim and reset the field together with clearing the Allocation field.

            While DeallocationRequested is set, no new consumers may be added to ReservedFor.
            """
            return self._set("deallocation_requested", value)

        def driver_name(self, value: Optional[str], /) -> Self:
            """
            DriverName is a copy of the driver name from the ResourceClass at the time when allocation started.
            """
            return self._set("driver_name", value)

        @overload
        def reserved_for(
            self, value_or_callback: List[ResourceClaimConsumerReference], /
        ) -> "ResourceClaimStatus.Builder": ...

        @overload
        def reserved_for(
            self,
            value_or_callback: Callable[
                [
                    GenericListBuilder[
                        ResourceClaimConsumerReference,
                        ResourceClaimConsumerReference.Builder,
                    ]
                ],
                GenericListBuilder[
                    ResourceClaimConsumerReference,
                    ResourceClaimConsumerReference.Builder,
                ]
                | List[ResourceClaimConsumerReference],
            ],
            /,
        ) -> "ResourceClaimStatus.Builder": ...

        @overload
        def reserved_for(
            self, value_or_callback: Never = ...
        ) -> ListBuilderContext[ResourceClaimConsumerReference.Builder]: ...

        def reserved_for(self, value_or_callback=None, /):
            """
            ReservedFor indicates which entities are currently allowed to use the claim. A Pod which references a ResourceClaim which is not reserved for that Pod will not be started.

            There can be at most 32 such reservations. This may get increased in the future, but not reduced.
            """
            if self._in_context and value_or_callback is None:
                context = ListBuilderContext[ResourceClaimConsumerReference.Builder]()
                context._parent_builder = self
                context._field_name = "reserved_for"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(ResourceClaimConsumerReference.list_builder())
                if isinstance(output, GenericListBuilder):
                    value = output.build()
                else:
                    value = output
            return self._set("reserved_for", value)

    class BuilderContext(BuilderContextBase["ResourceClaimStatus.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = ResourceClaimStatus.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for ResourceClaimStatus."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["ResourceClaimStatus", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use ResourceClaimStatus.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    allocation: Optional[AllocationResult] = None
    """
    Allocation is set by the resource driver once a resource or set of resources has been allocated successfully. If this is not specified, the resources have not been allocated yet.
    """
    deallocation_requested: Annotated[Optional[bool], Field(alias="deallocationRequested")] = None
    """
    DeallocationRequested indicates that a ResourceClaim is to be deallocated.

    The driver then must deallocate this claim and reset the field together with clearing the Allocation field.

    While DeallocationRequested is set, no new consumers may be added to ReservedFor.
    """
    driver_name: Annotated[Optional[str], Field(alias="driverName")] = None
    """
    DriverName is a copy of the driver name from the ResourceClass at the time when allocation started.
    """
    reserved_for: Annotated[
        Optional[List[ResourceClaimConsumerReference]], Field(alias="reservedFor")
    ] = None
    """
    ReservedFor indicates which entities are currently allowed to use the claim. A Pod which references a ResourceClaim which is not reserved for that Pod will not be started.

    There can be at most 32 such reservations. This may get increased in the future, but not reduced.
    """


class PodSchedulingContext(Resource):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["PodSchedulingContext"]:
            return PodSchedulingContext

        def build(self) -> "PodSchedulingContext":
            return PodSchedulingContext(**self._attrs)

        def api_version(self, value: Optional[Literal["resource.k8s.io/v1alpha2"]], /) -> Self:
            """
            APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
            """
            return self._set("api_version", value)

        def kind(self, value: Optional[Literal["PodSchedulingContext"]], /) -> Self:
            """
            Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
            """
            return self._set("kind", value)

        @overload
        def metadata(
            self, value_or_callback: Optional[apimachinery.ObjectMeta], /
        ) -> "PodSchedulingContext.Builder": ...

        @overload
        def metadata(
            self,
            value_or_callback: Callable[
                [apimachinery.ObjectMeta.Builder],
                apimachinery.ObjectMeta.Builder | apimachinery.ObjectMeta,
            ],
            /,
        ) -> "PodSchedulingContext.Builder": ...

        @overload
        def metadata(
            self, value_or_callback: Never = ...
        ) -> "apimachinery.ObjectMeta.BuilderContext": ...

        def metadata(self, value_or_callback=None, /):
            """
            Standard object metadata
            """
            if self._in_context and value_or_callback is None:
                context = apimachinery.ObjectMeta.BuilderContext()
                context._parent_builder = self
                context._field_name = "metadata"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(apimachinery.ObjectMeta.builder())
                if isinstance(output, apimachinery.ObjectMeta.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("metadata", value)

        @overload
        def spec(
            self, value_or_callback: PodSchedulingContextSpec, /
        ) -> "PodSchedulingContext.Builder": ...

        @overload
        def spec(
            self,
            value_or_callback: Callable[
                [PodSchedulingContextSpec.Builder],
                PodSchedulingContextSpec.Builder | PodSchedulingContextSpec,
            ],
            /,
        ) -> "PodSchedulingContext.Builder": ...

        @overload
        def spec(
            self, value_or_callback: Never = ...
        ) -> "PodSchedulingContextSpec.BuilderContext": ...

        def spec(self, value_or_callback=None, /):
            """
            Spec describes where resources for the Pod are needed.
            """
            if self._in_context and value_or_callback is None:
                context = PodSchedulingContextSpec.BuilderContext()
                context._parent_builder = self
                context._field_name = "spec"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(PodSchedulingContextSpec.builder())
                if isinstance(output, PodSchedulingContextSpec.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("spec", value)

        @overload
        def status(
            self, value_or_callback: Optional[PodSchedulingContextStatus], /
        ) -> "PodSchedulingContext.Builder": ...

        @overload
        def status(
            self,
            value_or_callback: Callable[
                [PodSchedulingContextStatus.Builder],
                PodSchedulingContextStatus.Builder | PodSchedulingContextStatus,
            ],
            /,
        ) -> "PodSchedulingContext.Builder": ...

        @overload
        def status(
            self, value_or_callback: Never = ...
        ) -> "PodSchedulingContextStatus.BuilderContext": ...

        def status(self, value_or_callback=None, /):
            """
            Status describes where resources for the Pod can be allocated.
            """
            if self._in_context and value_or_callback is None:
                context = PodSchedulingContextStatus.BuilderContext()
                context._parent_builder = self
                context._field_name = "status"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(PodSchedulingContextStatus.builder())
                if isinstance(output, PodSchedulingContextStatus.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("status", value)

    class BuilderContext(BuilderContextBase["PodSchedulingContext.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = PodSchedulingContext.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for PodSchedulingContext."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["PodSchedulingContext", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use PodSchedulingContext.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    api_version: Annotated[
        Optional[Literal["resource.k8s.io/v1alpha2"]], Field(alias="apiVersion")
    ] = "resource.k8s.io/v1alpha2"
    """
    APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
    """
    kind: Optional[Literal["PodSchedulingContext"]] = "PodSchedulingContext"
    """
    Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
    """
    metadata: Optional[apimachinery.ObjectMeta] = None
    """
    Standard object metadata
    """
    spec: PodSchedulingContextSpec
    """
    Spec describes where resources for the Pod are needed.
    """
    status: Optional[PodSchedulingContextStatus] = None
    """
    Status describes where resources for the Pod can be allocated.
    """


PodSchedulingContextList = ResourceList["PodSchedulingContext"]


class ResourceClaim(Resource):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["ResourceClaim"]:
            return ResourceClaim

        def build(self) -> "ResourceClaim":
            return ResourceClaim(**self._attrs)

        def api_version(self, value: Optional[Literal["resource.k8s.io/v1alpha2"]], /) -> Self:
            """
            APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
            """
            return self._set("api_version", value)

        def kind(self, value: Optional[Literal["ResourceClaim"]], /) -> Self:
            """
            Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
            """
            return self._set("kind", value)

        @overload
        def metadata(
            self, value_or_callback: Optional[apimachinery.ObjectMeta], /
        ) -> "ResourceClaim.Builder": ...

        @overload
        def metadata(
            self,
            value_or_callback: Callable[
                [apimachinery.ObjectMeta.Builder],
                apimachinery.ObjectMeta.Builder | apimachinery.ObjectMeta,
            ],
            /,
        ) -> "ResourceClaim.Builder": ...

        @overload
        def metadata(
            self, value_or_callback: Never = ...
        ) -> "apimachinery.ObjectMeta.BuilderContext": ...

        def metadata(self, value_or_callback=None, /):
            """
            Standard object metadata
            """
            if self._in_context and value_or_callback is None:
                context = apimachinery.ObjectMeta.BuilderContext()
                context._parent_builder = self
                context._field_name = "metadata"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(apimachinery.ObjectMeta.builder())
                if isinstance(output, apimachinery.ObjectMeta.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("metadata", value)

        @overload
        def spec(self, value_or_callback: ResourceClaimSpec, /) -> "ResourceClaim.Builder": ...

        @overload
        def spec(
            self,
            value_or_callback: Callable[
                [ResourceClaimSpec.Builder],
                ResourceClaimSpec.Builder | ResourceClaimSpec,
            ],
            /,
        ) -> "ResourceClaim.Builder": ...

        @overload
        def spec(self, value_or_callback: Never = ...) -> "ResourceClaimSpec.BuilderContext": ...

        def spec(self, value_or_callback=None, /):
            """
            Spec describes the desired attributes of a resource that then needs to be allocated. It can only be set once when creating the ResourceClaim.
            """
            if self._in_context and value_or_callback is None:
                context = ResourceClaimSpec.BuilderContext()
                context._parent_builder = self
                context._field_name = "spec"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(ResourceClaimSpec.builder())
                if isinstance(output, ResourceClaimSpec.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("spec", value)

        @overload
        def status(
            self, value_or_callback: Optional[ResourceClaimStatus], /
        ) -> "ResourceClaim.Builder": ...

        @overload
        def status(
            self,
            value_or_callback: Callable[
                [ResourceClaimStatus.Builder],
                ResourceClaimStatus.Builder | ResourceClaimStatus,
            ],
            /,
        ) -> "ResourceClaim.Builder": ...

        @overload
        def status(
            self, value_or_callback: Never = ...
        ) -> "ResourceClaimStatus.BuilderContext": ...

        def status(self, value_or_callback=None, /):
            """
            Status describes whether the resource is available and with which attributes.
            """
            if self._in_context and value_or_callback is None:
                context = ResourceClaimStatus.BuilderContext()
                context._parent_builder = self
                context._field_name = "status"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(ResourceClaimStatus.builder())
                if isinstance(output, ResourceClaimStatus.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("status", value)

    class BuilderContext(BuilderContextBase["ResourceClaim.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = ResourceClaim.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for ResourceClaim."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["ResourceClaim", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use ResourceClaim.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    api_version: Annotated[
        Optional[Literal["resource.k8s.io/v1alpha2"]], Field(alias="apiVersion")
    ] = "resource.k8s.io/v1alpha2"
    """
    APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
    """
    kind: Optional[Literal["ResourceClaim"]] = "ResourceClaim"
    """
    Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
    """
    metadata: Optional[apimachinery.ObjectMeta] = None
    """
    Standard object metadata
    """
    spec: ResourceClaimSpec
    """
    Spec describes the desired attributes of a resource that then needs to be allocated. It can only be set once when creating the ResourceClaim.
    """
    status: Optional[ResourceClaimStatus] = None
    """
    Status describes whether the resource is available and with which attributes.
    """


ResourceClaimList = ResourceList["ResourceClaim"]


class ResourceClaimTemplateSpec(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["ResourceClaimTemplateSpec"]:
            return ResourceClaimTemplateSpec

        def build(self) -> "ResourceClaimTemplateSpec":
            return ResourceClaimTemplateSpec(**self._attrs)

        @overload
        def metadata(
            self, value_or_callback: Optional[apimachinery.ObjectMeta], /
        ) -> "ResourceClaimTemplateSpec.Builder": ...

        @overload
        def metadata(
            self,
            value_or_callback: Callable[
                [apimachinery.ObjectMeta.Builder],
                apimachinery.ObjectMeta.Builder | apimachinery.ObjectMeta,
            ],
            /,
        ) -> "ResourceClaimTemplateSpec.Builder": ...

        @overload
        def metadata(
            self, value_or_callback: Never = ...
        ) -> "apimachinery.ObjectMeta.BuilderContext": ...

        def metadata(self, value_or_callback=None, /):
            """
            ObjectMeta may contain labels and annotations that will be copied into the PVC when creating it. No other fields are allowed and will be rejected during validation.
            """
            if self._in_context and value_or_callback is None:
                context = apimachinery.ObjectMeta.BuilderContext()
                context._parent_builder = self
                context._field_name = "metadata"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(apimachinery.ObjectMeta.builder())
                if isinstance(output, apimachinery.ObjectMeta.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("metadata", value)

        @overload
        def spec(
            self, value_or_callback: ResourceClaimSpec, /
        ) -> "ResourceClaimTemplateSpec.Builder": ...

        @overload
        def spec(
            self,
            value_or_callback: Callable[
                [ResourceClaimSpec.Builder],
                ResourceClaimSpec.Builder | ResourceClaimSpec,
            ],
            /,
        ) -> "ResourceClaimTemplateSpec.Builder": ...

        @overload
        def spec(self, value_or_callback: Never = ...) -> "ResourceClaimSpec.BuilderContext": ...

        def spec(self, value_or_callback=None, /):
            """
            Spec for the ResourceClaim. The entire content is copied unchanged into the ResourceClaim that gets created from this template. The same fields as in a ResourceClaim are also valid here.
            """
            if self._in_context and value_or_callback is None:
                context = ResourceClaimSpec.BuilderContext()
                context._parent_builder = self
                context._field_name = "spec"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(ResourceClaimSpec.builder())
                if isinstance(output, ResourceClaimSpec.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("spec", value)

    class BuilderContext(BuilderContextBase["ResourceClaimTemplateSpec.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = ResourceClaimTemplateSpec.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for ResourceClaimTemplateSpec."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["ResourceClaimTemplateSpec", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use ResourceClaimTemplateSpec.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    metadata: Optional[apimachinery.ObjectMeta] = None
    """
    ObjectMeta may contain labels and annotations that will be copied into the PVC when creating it. No other fields are allowed and will be rejected during validation.
    """
    spec: ResourceClaimSpec
    """
    Spec for the ResourceClaim. The entire content is copied unchanged into the ResourceClaim that gets created from this template. The same fields as in a ResourceClaim are also valid here.
    """


class ResourceClass(Resource):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["ResourceClass"]:
            return ResourceClass

        def build(self) -> "ResourceClass":
            return ResourceClass(**self._attrs)

        def api_version(self, value: Optional[Literal["resource.k8s.io/v1alpha2"]], /) -> Self:
            """
            APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
            """
            return self._set("api_version", value)

        def driver_name(self, value: str, /) -> Self:
            """
            DriverName defines the name of the dynamic resource driver that is used for allocation of a ResourceClaim that uses this class.

            Resource drivers have a unique name in forward domain order (acme.example.com).
            """
            return self._set("driver_name", value)

        def kind(self, value: Optional[Literal["ResourceClass"]], /) -> Self:
            """
            Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
            """
            return self._set("kind", value)

        @overload
        def metadata(
            self, value_or_callback: Optional[apimachinery.ObjectMeta], /
        ) -> "ResourceClass.Builder": ...

        @overload
        def metadata(
            self,
            value_or_callback: Callable[
                [apimachinery.ObjectMeta.Builder],
                apimachinery.ObjectMeta.Builder | apimachinery.ObjectMeta,
            ],
            /,
        ) -> "ResourceClass.Builder": ...

        @overload
        def metadata(
            self, value_or_callback: Never = ...
        ) -> "apimachinery.ObjectMeta.BuilderContext": ...

        def metadata(self, value_or_callback=None, /):
            """
            Standard object metadata
            """
            if self._in_context and value_or_callback is None:
                context = apimachinery.ObjectMeta.BuilderContext()
                context._parent_builder = self
                context._field_name = "metadata"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(apimachinery.ObjectMeta.builder())
                if isinstance(output, apimachinery.ObjectMeta.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("metadata", value)

        @overload
        def parameters_ref(
            self, value_or_callback: Optional[ResourceClassParametersReference], /
        ) -> "ResourceClass.Builder": ...

        @overload
        def parameters_ref(
            self,
            value_or_callback: Callable[
                [ResourceClassParametersReference.Builder],
                ResourceClassParametersReference.Builder | ResourceClassParametersReference,
            ],
            /,
        ) -> "ResourceClass.Builder": ...

        @overload
        def parameters_ref(
            self, value_or_callback: Never = ...
        ) -> "ResourceClassParametersReference.BuilderContext": ...

        def parameters_ref(self, value_or_callback=None, /):
            """
            ParametersRef references an arbitrary separate object that may hold parameters that will be used by the driver when allocating a resource that uses this class. A dynamic resource driver can distinguish between parameters stored here and and those stored in ResourceClaimSpec.
            """
            if self._in_context and value_or_callback is None:
                context = ResourceClassParametersReference.BuilderContext()
                context._parent_builder = self
                context._field_name = "parameters_ref"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(ResourceClassParametersReference.builder())
                if isinstance(output, ResourceClassParametersReference.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("parameters_ref", value)

        @overload
        def suitable_nodes(
            self, value_or_callback: Optional[v1.NodeSelector], /
        ) -> "ResourceClass.Builder": ...

        @overload
        def suitable_nodes(
            self,
            value_or_callback: Callable[
                [v1.NodeSelector.Builder], v1.NodeSelector.Builder | v1.NodeSelector
            ],
            /,
        ) -> "ResourceClass.Builder": ...

        @overload
        def suitable_nodes(
            self, value_or_callback: Never = ...
        ) -> "v1.NodeSelector.BuilderContext": ...

        def suitable_nodes(self, value_or_callback=None, /):
            """
            Only nodes matching the selector will be considered by the scheduler when trying to find a Node that fits a Pod when that Pod uses a ResourceClaim that has not been allocated yet.

            Setting this field is optional. If null, all nodes are candidates.
            """
            if self._in_context and value_or_callback is None:
                context = v1.NodeSelector.BuilderContext()
                context._parent_builder = self
                context._field_name = "suitable_nodes"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(v1.NodeSelector.builder())
                if isinstance(output, v1.NodeSelector.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("suitable_nodes", value)

    class BuilderContext(BuilderContextBase["ResourceClass.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = ResourceClass.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for ResourceClass."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["ResourceClass", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use ResourceClass.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    api_version: Annotated[
        Optional[Literal["resource.k8s.io/v1alpha2"]], Field(alias="apiVersion")
    ] = "resource.k8s.io/v1alpha2"
    """
    APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
    """
    driver_name: Annotated[str, Field(alias="driverName")]
    """
    DriverName defines the name of the dynamic resource driver that is used for allocation of a ResourceClaim that uses this class.

    Resource drivers have a unique name in forward domain order (acme.example.com).
    """
    kind: Optional[Literal["ResourceClass"]] = "ResourceClass"
    """
    Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
    """
    metadata: Optional[apimachinery.ObjectMeta] = None
    """
    Standard object metadata
    """
    parameters_ref: Annotated[
        Optional[ResourceClassParametersReference], Field(alias="parametersRef")
    ] = None
    """
    ParametersRef references an arbitrary separate object that may hold parameters that will be used by the driver when allocating a resource that uses this class. A dynamic resource driver can distinguish between parameters stored here and and those stored in ResourceClaimSpec.
    """
    suitable_nodes: Annotated[Optional[v1.NodeSelector], Field(alias="suitableNodes")] = None
    """
    Only nodes matching the selector will be considered by the scheduler when trying to find a Node that fits a Pod when that Pod uses a ResourceClaim that has not been allocated yet.

    Setting this field is optional. If null, all nodes are candidates.
    """


ResourceClassList = ResourceList["ResourceClass"]


class ResourceClaimTemplate(Resource):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["ResourceClaimTemplate"]:
            return ResourceClaimTemplate

        def build(self) -> "ResourceClaimTemplate":
            return ResourceClaimTemplate(**self._attrs)

        def api_version(self, value: Optional[Literal["resource.k8s.io/v1alpha2"]], /) -> Self:
            """
            APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
            """
            return self._set("api_version", value)

        def kind(self, value: Optional[Literal["ResourceClaimTemplate"]], /) -> Self:
            """
            Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
            """
            return self._set("kind", value)

        @overload
        def metadata(
            self, value_or_callback: Optional[apimachinery.ObjectMeta], /
        ) -> "ResourceClaimTemplate.Builder": ...

        @overload
        def metadata(
            self,
            value_or_callback: Callable[
                [apimachinery.ObjectMeta.Builder],
                apimachinery.ObjectMeta.Builder | apimachinery.ObjectMeta,
            ],
            /,
        ) -> "ResourceClaimTemplate.Builder": ...

        @overload
        def metadata(
            self, value_or_callback: Never = ...
        ) -> "apimachinery.ObjectMeta.BuilderContext": ...

        def metadata(self, value_or_callback=None, /):
            """
            Standard object metadata
            """
            if self._in_context and value_or_callback is None:
                context = apimachinery.ObjectMeta.BuilderContext()
                context._parent_builder = self
                context._field_name = "metadata"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(apimachinery.ObjectMeta.builder())
                if isinstance(output, apimachinery.ObjectMeta.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("metadata", value)

        @overload
        def spec(
            self, value_or_callback: ResourceClaimTemplateSpec, /
        ) -> "ResourceClaimTemplate.Builder": ...

        @overload
        def spec(
            self,
            value_or_callback: Callable[
                [ResourceClaimTemplateSpec.Builder],
                ResourceClaimTemplateSpec.Builder | ResourceClaimTemplateSpec,
            ],
            /,
        ) -> "ResourceClaimTemplate.Builder": ...

        @overload
        def spec(
            self, value_or_callback: Never = ...
        ) -> "ResourceClaimTemplateSpec.BuilderContext": ...

        def spec(self, value_or_callback=None, /):
            """
            Describes the ResourceClaim that is to be generated.

            This field is immutable. A ResourceClaim will get created by the control plane for a Pod when needed and then not get updated anymore.
            """
            if self._in_context and value_or_callback is None:
                context = ResourceClaimTemplateSpec.BuilderContext()
                context._parent_builder = self
                context._field_name = "spec"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(ResourceClaimTemplateSpec.builder())
                if isinstance(output, ResourceClaimTemplateSpec.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("spec", value)

    class BuilderContext(BuilderContextBase["ResourceClaimTemplate.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = ResourceClaimTemplate.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for ResourceClaimTemplate."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["ResourceClaimTemplate", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use ResourceClaimTemplate.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    api_version: Annotated[
        Optional[Literal["resource.k8s.io/v1alpha2"]], Field(alias="apiVersion")
    ] = "resource.k8s.io/v1alpha2"
    """
    APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
    """
    kind: Optional[Literal["ResourceClaimTemplate"]] = "ResourceClaimTemplate"
    """
    Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
    """
    metadata: Optional[apimachinery.ObjectMeta] = None
    """
    Standard object metadata
    """
    spec: ResourceClaimTemplateSpec
    """
    Describes the ResourceClaim that is to be generated.

    This field is immutable. A ResourceClaim will get created by the control plane for a Pod when needed and then not get updated anymore.
    """


ResourceClaimTemplateList = ResourceList["ResourceClaimTemplate"]



# Generated by cloudcoil-model-codegen v0.3.0
# DO NOT EDIT

from __future__ import annotations

from typing import (
    Annotated,
    Callable,
    Dict,
    List,
    Literal,
    Optional,
    Type,
    overload,
)

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil.pydantic import (
    BaseModel,
    BaseModelBuilder,
    BuilderContextBase,
    GenericListBuilder,
    ListBuilderContext,
    Never,
    Self,
)
from cloudcoil.resources import Resource, ResourceList


class Eviction(Resource):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["Eviction"]:
            return Eviction

        def build(self) -> "Eviction":
            return Eviction(**self._attrs)

        def api_version(self, value: Optional[Literal["policy/v1"]], /) -> Self:
            """
            APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
            """
            return self._set("api_version", value)

        @overload
        def delete_options(
            self, value_or_callback: Optional[apimachinery.DeleteOptions], /
        ) -> "Eviction.Builder": ...

        @overload
        def delete_options(
            self,
            value_or_callback: Callable[
                [apimachinery.DeleteOptions.Builder],
                apimachinery.DeleteOptions.Builder | apimachinery.DeleteOptions,
            ],
            /,
        ) -> "Eviction.Builder": ...

        @overload
        def delete_options(
            self, value_or_callback: Never = ...
        ) -> "apimachinery.DeleteOptions.BuilderContext": ...

        def delete_options(self, value_or_callback=None, /):
            """
            DeleteOptions may be provided
            """
            if self._in_context and value_or_callback is None:
                context = apimachinery.DeleteOptions.BuilderContext()
                context._parent_builder = self
                context._field_name = "delete_options"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(apimachinery.DeleteOptions.builder())
                if isinstance(output, apimachinery.DeleteOptions.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("delete_options", value)

        def kind(self, value: Optional[Literal["Eviction"]], /) -> Self:
            """
            Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
            """
            return self._set("kind", value)

        @overload
        def metadata(
            self, value_or_callback: Optional[apimachinery.ObjectMeta], /
        ) -> "Eviction.Builder": ...

        @overload
        def metadata(
            self,
            value_or_callback: Callable[
                [apimachinery.ObjectMeta.Builder],
                apimachinery.ObjectMeta.Builder | apimachinery.ObjectMeta,
            ],
            /,
        ) -> "Eviction.Builder": ...

        @overload
        def metadata(
            self, value_or_callback: Never = ...
        ) -> "apimachinery.ObjectMeta.BuilderContext": ...

        def metadata(self, value_or_callback=None, /):
            """
            ObjectMeta describes the pod that is being evicted.
            """
            if self._in_context and value_or_callback is None:
                context = apimachinery.ObjectMeta.BuilderContext()
                context._parent_builder = self
                context._field_name = "metadata"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(apimachinery.ObjectMeta.builder())
                if isinstance(output, apimachinery.ObjectMeta.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("metadata", value)

    class BuilderContext(BuilderContextBase["Eviction.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = Eviction.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for Eviction."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["Eviction", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use Eviction.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    api_version: Annotated[Optional[Literal["policy/v1"]], Field(alias="apiVersion")] = "policy/v1"
    """
    APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
    """
    delete_options: Annotated[
        Optional[apimachinery.DeleteOptions], Field(alias="deleteOptions")
    ] = None
    """
    DeleteOptions may be provided
    """
    kind: Optional[Literal["Eviction"]] = "Eviction"
    """
    Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
    """
    metadata: Optional[apimachinery.ObjectMeta] = None
    """
    ObjectMeta describes the pod that is being evicted.
    """


class PodDisruptionBudgetSpec(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["PodDisruptionBudgetSpec"]:
            return PodDisruptionBudgetSpec

        def build(self) -> "PodDisruptionBudgetSpec":
            return PodDisruptionBudgetSpec(**self._attrs)

        @overload
        def max_unavailable(
            self, value_or_callback: Optional[apimachinery.IntOrString], /
        ) -> "PodDisruptionBudgetSpec.Builder": ...

        @overload
        def max_unavailable(
            self,
            value_or_callback: Callable[
                [apimachinery.IntOrString.Builder],
                apimachinery.IntOrString.Builder | apimachinery.IntOrString,
            ],
            /,
        ) -> "PodDisruptionBudgetSpec.Builder": ...

        @overload
        def max_unavailable(
            self, value_or_callback: Never = ...
        ) -> "apimachinery.IntOrString.BuilderContext": ...

        def max_unavailable(self, value_or_callback=None, /):
            """
            An eviction is allowed if at most "maxUnavailable" pods selected by "selector" are unavailable after the eviction, i.e. even in absence of the evicted pod. For example, one can prevent all voluntary evictions by specifying 0. This is a mutually exclusive setting with "minAvailable".
            """
            if self._in_context and value_or_callback is None:
                context = apimachinery.IntOrString.BuilderContext()
                context._parent_builder = self
                context._field_name = "max_unavailable"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(apimachinery.IntOrString.builder())
                if isinstance(output, apimachinery.IntOrString.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("max_unavailable", value)

        @overload
        def min_available(
            self, value_or_callback: Optional[apimachinery.IntOrString], /
        ) -> "PodDisruptionBudgetSpec.Builder": ...

        @overload
        def min_available(
            self,
            value_or_callback: Callable[
                [apimachinery.IntOrString.Builder],
                apimachinery.IntOrString.Builder | apimachinery.IntOrString,
            ],
            /,
        ) -> "PodDisruptionBudgetSpec.Builder": ...

        @overload
        def min_available(
            self, value_or_callback: Never = ...
        ) -> "apimachinery.IntOrString.BuilderContext": ...

        def min_available(self, value_or_callback=None, /):
            """
            An eviction is allowed if at least "minAvailable" pods selected by "selector" will still be available after the eviction, i.e. even in the absence of the evicted pod.  So for example you can prevent all voluntary evictions by specifying "100%".
            """
            if self._in_context and value_or_callback is None:
                context = apimachinery.IntOrString.BuilderContext()
                context._parent_builder = self
                context._field_name = "min_available"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(apimachinery.IntOrString.builder())
                if isinstance(output, apimachinery.IntOrString.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("min_available", value)

        @overload
        def selector(
            self, value_or_callback: Optional[apimachinery.LabelSelector], /
        ) -> "PodDisruptionBudgetSpec.Builder": ...

        @overload
        def selector(
            self,
            value_or_callback: Callable[
                [apimachinery.LabelSelector.Builder],
                apimachinery.LabelSelector.Builder | apimachinery.LabelSelector,
            ],
            /,
        ) -> "PodDisruptionBudgetSpec.Builder": ...

        @overload
        def selector(
            self, value_or_callback: Never = ...
        ) -> "apimachinery.LabelSelector.BuilderContext": ...

        def selector(self, value_or_callback=None, /):
            """
            Label query over pods whose evictions are managed by the disruption budget. A null selector will match no pods, while an empty ({}) selector will select all pods within the namespace.
            """
            if self._in_context and value_or_callback is None:
                context = apimachinery.LabelSelector.BuilderContext()
                context._parent_builder = self
                context._field_name = "selector"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(apimachinery.LabelSelector.builder())
                if isinstance(output, apimachinery.LabelSelector.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("selector", value)

        def unhealthy_pod_eviction_policy(self, value: Optional[str], /) -> Self:
            """
            UnhealthyPodEvictionPolicy defines the criteria for when unhealthy pods should be considered for eviction. Current implementation considers healthy pods, as pods that have status.conditions item with type="Ready",status="True".

            Valid policies are IfHealthyBudget and AlwaysAllow. If no policy is specified, the default behavior will be used, which corresponds to the IfHealthyBudget policy.

            IfHealthyBudget policy means that running pods (status.phase="Running"), but not yet healthy can be evicted only if the guarded application is not disrupted (status.currentHealthy is at least equal to status.desiredHealthy). Healthy pods will be subject to the PDB for eviction.

            AlwaysAllow policy means that all running pods (status.phase="Running"), but not yet healthy are considered disrupted and can be evicted regardless of whether the criteria in a PDB is met. This means perspective running pods of a disrupted application might not get a chance to become healthy. Healthy pods will be subject to the PDB for eviction.

            Additional policies may be added in the future. Clients making eviction decisions should disallow eviction of unhealthy pods if they encounter an unrecognized policy in this field.

            This field is beta-level. The eviction API uses this field when the feature gate PDBUnhealthyPodEvictionPolicy is enabled (enabled by default).
            """
            return self._set("unhealthy_pod_eviction_policy", value)

    class BuilderContext(BuilderContextBase["PodDisruptionBudgetSpec.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = PodDisruptionBudgetSpec.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for PodDisruptionBudgetSpec."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["PodDisruptionBudgetSpec", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use PodDisruptionBudgetSpec.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    max_unavailable: Annotated[
        Optional[apimachinery.IntOrString], Field(alias="maxUnavailable")
    ] = None
    """
    An eviction is allowed if at most "maxUnavailable" pods selected by "selector" are unavailable after the eviction, i.e. even in absence of the evicted pod. For example, one can prevent all voluntary evictions by specifying 0. This is a mutually exclusive setting with "minAvailable".
    """
    min_available: Annotated[Optional[apimachinery.IntOrString], Field(alias="minAvailable")] = None
    """
    An eviction is allowed if at least "minAvailable" pods selected by "selector" will still be available after the eviction, i.e. even in the absence of the evicted pod.  So for example you can prevent all voluntary evictions by specifying "100%".
    """
    selector: Optional[apimachinery.LabelSelector] = None
    """
    Label query over pods whose evictions are managed by the disruption budget. A null selector will match no pods, while an empty ({}) selector will select all pods within the namespace.
    """
    unhealthy_pod_eviction_policy: Annotated[
        Optional[str], Field(alias="unhealthyPodEvictionPolicy")
    ] = None
    """
    UnhealthyPodEvictionPolicy defines the criteria for when unhealthy pods should be considered for eviction. Current implementation considers healthy pods, as pods that have status.conditions item with type="Ready",status="True".

    Valid policies are IfHealthyBudget and AlwaysAllow. If no policy is specified, the default behavior will be used, which corresponds to the IfHealthyBudget policy.

    IfHealthyBudget policy means that running pods (status.phase="Running"), but not yet healthy can be evicted only if the guarded application is not disrupted (status.currentHealthy is at least equal to status.desiredHealthy). Healthy pods will be subject to the PDB for eviction.

    AlwaysAllow policy means that all running pods (status.phase="Running"), but not yet healthy are considered disrupted and can be evicted regardless of whether the criteria in a PDB is met. This means perspective running pods of a disrupted application might not get a chance to become healthy. Healthy pods will be subject to the PDB for eviction.

    Additional policies may be added in the future. Clients making eviction decisions should disallow eviction of unhealthy pods if they encounter an unrecognized policy in this field.

    This field is beta-level. The eviction API uses this field when the feature gate PDBUnhealthyPodEvictionPolicy is enabled (enabled by default).
    """


class PodDisruptionBudgetStatus(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["PodDisruptionBudgetStatus"]:
            return PodDisruptionBudgetStatus

        def build(self) -> "PodDisruptionBudgetStatus":
            return PodDisruptionBudgetStatus(**self._attrs)

        @overload
        def conditions(
            self, value_or_callback: List[apimachinery.Condition], /
        ) -> "PodDisruptionBudgetStatus.Builder": ...

        @overload
        def conditions(
            self,
            value_or_callback: Callable[
                [GenericListBuilder[apimachinery.Condition, apimachinery.Condition.Builder]],
                GenericListBuilder[apimachinery.Condition, apimachinery.Condition.Builder]
                | List[apimachinery.Condition],
            ],
            /,
        ) -> "PodDisruptionBudgetStatus.Builder": ...

        @overload
        def conditions(
            self, value_or_callback: Never = ...
        ) -> ListBuilderContext[apimachinery.Condition.Builder]: ...

        def conditions(self, value_or_callback=None, /):
            """
            Conditions contain conditions for PDB. The disruption controller sets the DisruptionAllowed condition. The following are known values for the reason field (additional reasons could be added in the future): - SyncFailed: The controller encountered an error and wasn't able to compute
                          the number of allowed disruptions. Therefore no disruptions are
                          allowed and the status of the condition will be False.
            - InsufficientPods: The number of pods are either at or below the number
                                required by the PodDisruptionBudget. No disruptions are
                                allowed and the status of the condition will be False.
            - SufficientPods: There are more pods than required by the PodDisruptionBudget.
                              The condition will be True, and the number of allowed
                              disruptions are provided by the disruptionsAllowed property.
            """
            if self._in_context and value_or_callback is None:
                context = ListBuilderContext[apimachinery.Condition.Builder]()
                context._parent_builder = self
                context._field_name = "conditions"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(apimachinery.Condition.list_builder())
                if isinstance(output, GenericListBuilder):
                    value = output.build()
                else:
                    value = output
            return self._set("conditions", value)

        def current_healthy(self, value: int, /) -> Self:
            """
            current number of healthy pods
            """
            return self._set("current_healthy", value)

        def desired_healthy(self, value: int, /) -> Self:
            """
            minimum desired number of healthy pods
            """
            return self._set("desired_healthy", value)

        def disrupted_pods(self, value: Optional[Dict[str, apimachinery.Time]], /) -> Self:
            """
            DisruptedPods contains information about pods whose eviction was processed by the API server eviction subresource handler but has not yet been observed by the PodDisruptionBudget controller. A pod will be in this map from the time when the API server processed the eviction request to the time when the pod is seen by PDB controller as having been marked for deletion (or after a timeout). The key in the map is the name of the pod and the value is the time when the API server processed the eviction request. If the deletion didn't occur and a pod is still there it will be removed from the list automatically by PodDisruptionBudget controller after some time. If everything goes smooth this map should be empty for the most of the time. Large number of entries in the map may indicate problems with pod deletions.
            """
            return self._set("disrupted_pods", value)

        def disruptions_allowed(self, value: int, /) -> Self:
            """
            Number of pod disruptions that are currently allowed.
            """
            return self._set("disruptions_allowed", value)

        def expected_pods(self, value: int, /) -> Self:
            """
            total number of pods counted by this disruption budget
            """
            return self._set("expected_pods", value)

        def observed_generation(self, value: Optional[int], /) -> Self:
            """
            Most recent generation observed when updating this PDB status. DisruptionsAllowed and other status information is valid only if observedGeneration equals to PDB's object generation.
            """
            return self._set("observed_generation", value)

    class BuilderContext(BuilderContextBase["PodDisruptionBudgetStatus.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = PodDisruptionBudgetStatus.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for PodDisruptionBudgetStatus."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["PodDisruptionBudgetStatus", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use PodDisruptionBudgetStatus.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    conditions: Optional[List[apimachinery.Condition]] = None
    """
    Conditions contain conditions for PDB. The disruption controller sets the DisruptionAllowed condition. The following are known values for the reason field (additional reasons could be added in the future): - SyncFailed: The controller encountered an error and wasn't able to compute
                  the number of allowed disruptions. Therefore no disruptions are
                  allowed and the status of the condition will be False.
    - InsufficientPods: The number of pods are either at or below the number
                        required by the PodDisruptionBudget. No disruptions are
                        allowed and the status of the condition will be False.
    - SufficientPods: There are more pods than required by the PodDisruptionBudget.
                      The condition will be True, and the number of allowed
                      disruptions are provided by the disruptionsAllowed property.
    """
    current_healthy: Annotated[int, Field(alias="currentHealthy")]
    """
    current number of healthy pods
    """
    desired_healthy: Annotated[int, Field(alias="desiredHealthy")]
    """
    minimum desired number of healthy pods
    """
    disrupted_pods: Annotated[
        Optional[Dict[str, apimachinery.Time]], Field(alias="disruptedPods")
    ] = None
    """
    DisruptedPods contains information about pods whose eviction was processed by the API server eviction subresource handler but has not yet been observed by the PodDisruptionBudget controller. A pod will be in this map from the time when the API server processed the eviction request to the time when the pod is seen by PDB controller as having been marked for deletion (or after a timeout). The key in the map is the name of the pod and the value is the time when the API server processed the eviction request. If the deletion didn't occur and a pod is still there it will be removed from the list automatically by PodDisruptionBudget controller after some time. If everything goes smooth this map should be empty for the most of the time. Large number of entries in the map may indicate problems with pod deletions.
    """
    disruptions_allowed: Annotated[int, Field(alias="disruptionsAllowed")]
    """
    Number of pod disruptions that are currently allowed.
    """
    expected_pods: Annotated[int, Field(alias="expectedPods")]
    """
    total number of pods counted by this disruption budget
    """
    observed_generation: Annotated[Optional[int], Field(alias="observedGeneration")] = None
    """
    Most recent generation observed when updating this PDB status. DisruptionsAllowed and other status information is valid only if observedGeneration equals to PDB's object generation.
    """


class PodDisruptionBudget(Resource):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["PodDisruptionBudget"]:
            return PodDisruptionBudget

        def build(self) -> "PodDisruptionBudget":
            return PodDisruptionBudget(**self._attrs)

        def api_version(self, value: Optional[Literal["policy/v1"]], /) -> Self:
            """
            APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
            """
            return self._set("api_version", value)

        def kind(self, value: Optional[Literal["PodDisruptionBudget"]], /) -> Self:
            """
            Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
            """
            return self._set("kind", value)

        @overload
        def metadata(
            self, value_or_callback: Optional[apimachinery.ObjectMeta], /
        ) -> "PodDisruptionBudget.Builder": ...

        @overload
        def metadata(
            self,
            value_or_callback: Callable[
                [apimachinery.ObjectMeta.Builder],
                apimachinery.ObjectMeta.Builder | apimachinery.ObjectMeta,
            ],
            /,
        ) -> "PodDisruptionBudget.Builder": ...

        @overload
        def metadata(
            self, value_or_callback: Never = ...
        ) -> "apimachinery.ObjectMeta.BuilderContext": ...

        def metadata(self, value_or_callback=None, /):
            """
            Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
            """
            if self._in_context and value_or_callback is None:
                context = apimachinery.ObjectMeta.BuilderContext()
                context._parent_builder = self
                context._field_name = "metadata"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(apimachinery.ObjectMeta.builder())
                if isinstance(output, apimachinery.ObjectMeta.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("metadata", value)

        @overload
        def spec(
            self, value_or_callback: Optional[PodDisruptionBudgetSpec], /
        ) -> "PodDisruptionBudget.Builder": ...

        @overload
        def spec(
            self,
            value_or_callback: Callable[
                [PodDisruptionBudgetSpec.Builder],
                PodDisruptionBudgetSpec.Builder | PodDisruptionBudgetSpec,
            ],
            /,
        ) -> "PodDisruptionBudget.Builder": ...

        @overload
        def spec(
            self, value_or_callback: Never = ...
        ) -> "PodDisruptionBudgetSpec.BuilderContext": ...

        def spec(self, value_or_callback=None, /):
            """
            Specification of the desired behavior of the PodDisruptionBudget.
            """
            if self._in_context and value_or_callback is None:
                context = PodDisruptionBudgetSpec.BuilderContext()
                context._parent_builder = self
                context._field_name = "spec"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(PodDisruptionBudgetSpec.builder())
                if isinstance(output, PodDisruptionBudgetSpec.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("spec", value)

        @overload
        def status(
            self, value_or_callback: Optional[PodDisruptionBudgetStatus], /
        ) -> "PodDisruptionBudget.Builder": ...

        @overload
        def status(
            self,
            value_or_callback: Callable[
                [PodDisruptionBudgetStatus.Builder],
                PodDisruptionBudgetStatus.Builder | PodDisruptionBudgetStatus,
            ],
            /,
        ) -> "PodDisruptionBudget.Builder": ...

        @overload
        def status(
            self, value_or_callback: Never = ...
        ) -> "PodDisruptionBudgetStatus.BuilderContext": ...

        def status(self, value_or_callback=None, /):
            """
            Most recently observed status of the PodDisruptionBudget.
            """
            if self._in_context and value_or_callback is None:
                context = PodDisruptionBudgetStatus.BuilderContext()
                context._parent_builder = self
                context._field_name = "status"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(PodDisruptionBudgetStatus.builder())
                if isinstance(output, PodDisruptionBudgetStatus.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("status", value)

    class BuilderContext(BuilderContextBase["PodDisruptionBudget.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = PodDisruptionBudget.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for PodDisruptionBudget."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["PodDisruptionBudget", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use PodDisruptionBudget.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    api_version: Annotated[Optional[Literal["policy/v1"]], Field(alias="apiVersion")] = "policy/v1"
    """
    APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
    """
    kind: Optional[Literal["PodDisruptionBudget"]] = "PodDisruptionBudget"
    """
    Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
    """
    metadata: Optional[apimachinery.ObjectMeta] = None
    """
    Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
    """
    spec: Optional[PodDisruptionBudgetSpec] = None
    """
    Specification of the desired behavior of the PodDisruptionBudget.
    """
    status: Optional[PodDisruptionBudgetStatus] = None
    """
    Most recently observed status of the PodDisruptionBudget.
    """


PodDisruptionBudgetList = ResourceList["PodDisruptionBudget"]



# Generated by cloudcoil-model-codegen v0.3.0
# DO NOT EDIT

from __future__ import annotations

from typing import (
    Annotated,
    Callable,
    List,
    Literal,
    Optional,
    Type,
    overload,
)

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil.pydantic import (
    BaseModel,
    BaseModelBuilder,
    BuilderContextBase,
    GenericListBuilder,
    ListBuilderContext,
    Never,
    Self,
)
from cloudcoil.resources import Resource, ResourceList


class PolicyRule(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["PolicyRule"]:
            return PolicyRule

        def build(self) -> "PolicyRule":
            return PolicyRule(**self._attrs)

        def api_groups(self, value: Optional[List[str]], /) -> Self:
            """
            APIGroups is the name of the APIGroup that contains the resources.  If multiple API groups are specified, any action requested against one of the enumerated resources in any API group will be allowed. "" represents the core API group and "*" represents all API groups.
            """
            return self._set("api_groups", value)

        def non_resource_ur_ls(self, value: Optional[List[str]], /) -> Self:
            """
            NonResourceURLs is a set of partial urls that a user should have access to.  *s are allowed, but only as the full, final step in the path Since non-resource URLs are not namespaced, this field is only applicable for ClusterRoles referenced from a ClusterRoleBinding. Rules can either apply to API resources (such as "pods" or "secrets") or non-resource URL paths (such as "/api"),  but not both.
            """
            return self._set("non_resource_ur_ls", value)

        def resource_names(self, value: Optional[List[str]], /) -> Self:
            """
            ResourceNames is an optional white list of names that the rule applies to.  An empty set means that everything is allowed.
            """
            return self._set("resource_names", value)

        def resources(self, value: Optional[List[str]], /) -> Self:
            """
            Resources is a list of resources this rule applies to. '*' represents all resources.
            """
            return self._set("resources", value)

        def verbs(self, value: List[str], /) -> Self:
            """
            Verbs is a list of Verbs that apply to ALL the ResourceKinds contained in this rule. '*' represents all verbs.
            """
            return self._set("verbs", value)

    class BuilderContext(BuilderContextBase["PolicyRule.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = PolicyRule.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for PolicyRule."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["PolicyRule", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use PolicyRule.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    api_groups: Annotated[Optional[List[str]], Field(alias="apiGroups")] = None
    """
    APIGroups is the name of the APIGroup that contains the resources.  If multiple API groups are specified, any action requested against one of the enumerated resources in any API group will be allowed. "" represents the core API group and "*" represents all API groups.
    """
    non_resource_ur_ls: Annotated[Optional[List[str]], Field(alias="nonResourceURLs")] = None
    """
    NonResourceURLs is a set of partial urls that a user should have access to.  *s are allowed, but only as the full, final step in the path Since non-resource URLs are not namespaced, this field is only applicable for ClusterRoles referenced from a ClusterRoleBinding. Rules can either apply to API resources (such as "pods" or "secrets") or non-resource URL paths (such as "/api"),  but not both.
    """
    resource_names: Annotated[Optional[List[str]], Field(alias="resourceNames")] = None
    """
    ResourceNames is an optional white list of names that the rule applies to.  An empty set means that everything is allowed.
    """
    resources: Optional[List[str]] = None
    """
    Resources is a list of resources this rule applies to. '*' represents all resources.
    """
    verbs: List[str]
    """
    Verbs is a list of Verbs that apply to ALL the ResourceKinds contained in this rule. '*' represents all verbs.
    """


class RoleRef(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["RoleRef"]:
            return RoleRef

        def build(self) -> "RoleRef":
            return RoleRef(**self._attrs)

        def api_group(self, value: str, /) -> Self:
            """
            APIGroup is the group for the resource being referenced
            """
            return self._set("api_group", value)

        def kind(self, value: str, /) -> Self:
            """
            Kind is the type of resource being referenced
            """
            return self._set("kind", value)

        def name(self, value: str, /) -> Self:
            """
            Name is the name of resource being referenced
            """
            return self._set("name", value)

    class BuilderContext(BuilderContextBase["RoleRef.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = RoleRef.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for RoleRef."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["RoleRef", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use RoleRef.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    api_group: Annotated[str, Field(alias="apiGroup")]
    """
    APIGroup is the group for the resource being referenced
    """
    kind: str
    """
    Kind is the type of resource being referenced
    """
    name: str
    """
    Name is the name of resource being referenced
    """


class Subject(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["Subject"]:
            return Subject

        def build(self) -> "Subject":
            return Subject(**self._attrs)

        def api_group(self, value: Optional[str], /) -> Self:
            """
            APIGroup holds the API group of the referenced subject. Defaults to "" for ServiceAccount subjects. Defaults to "rbac.authorization.k8s.io" for User and Group subjects.
            """
            return self._set("api_group", value)

        def kind(self, value: str, /) -> Self:
            """
            Kind of object being referenced. Values defined by this API group are "User", "Group", and "ServiceAccount". If the Authorizer does not recognized the kind value, the Authorizer should report an error.
            """
            return self._set("kind", value)

        def name(self, value: str, /) -> Self:
            """
            Name of the object being referenced.
            """
            return self._set("name", value)

        def namespace(self, value: Optional[str], /) -> Self:
            """
            Namespace of the referenced object.  If the object kind is non-namespace, such as "User" or "Group", and this value is not empty the Authorizer should report an error.
            """
            return self._set("namespace", value)

    class BuilderContext(BuilderContextBase["Subject.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = Subject.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for Subject."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["Subject", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use Subject.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    api_group: Annotated[Optional[str], Field(alias="apiGroup")] = None
    """
    APIGroup holds the API group of the referenced subject. Defaults to "" for ServiceAccount subjects. Defaults to "rbac.authorization.k8s.io" for User and Group subjects.
    """
    kind: str
    """
    Kind of object being referenced. Values defined by this API group are "User", "Group", and "ServiceAccount". If the Authorizer does not recognized the kind value, the Authorizer should report an error.
    """
    name: str
    """
    Name of the object being referenced.
    """
    namespace: Optional[str] = None
    """
    Namespace of the referenced object.  If the object kind is non-namespace, such as "User" or "Group", and this value is not empty the Authorizer should report an error.
    """


class AggregationRule(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["AggregationRule"]:
            return AggregationRule

        def build(self) -> "AggregationRule":
            return AggregationRule(**self._attrs)

        @overload
        def cluster_role_selectors(
            self, value_or_callback: List[apimachinery.LabelSelector], /
        ) -> "AggregationRule.Builder": ...

        @overload
        def cluster_role_selectors(
            self,
            value_or_callback: Callable[
                [
                    GenericListBuilder[
                        apimachinery.LabelSelector, apimachinery.LabelSelector.Builder
                    ]
                ],
                GenericListBuilder[apimachinery.LabelSelector, apimachinery.LabelSelector.Builder]
                | List[apimachinery.LabelSelector],
            ],
            /,
        ) -> "AggregationRule.Builder": ...

        @overload
        def cluster_role_selectors(
            self, value_or_callback: Never = ...
        ) -> ListBuilderContext[apimachinery.LabelSelector.Builder]: ...

        def cluster_role_selectors(self, value_or_callback=None, /):
            """
            ClusterRoleSelectors holds a list of selectors which will be used to find ClusterRoles and create the rules. If any of the selectors match, then the ClusterRole's permissions will be added
            """
            if self._in_context and value_or_callback is None:
                context = ListBuilderContext[apimachinery.LabelSelector.Builder]()
                context._parent_builder = self
                context._field_name = "cluster_role_selectors"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(apimachinery.LabelSelector.list_builder())
                if isinstance(output, GenericListBuilder):
                    value = output.build()
                else:
                    value = output
            return self._set("cluster_role_selectors", value)

    class BuilderContext(BuilderContextBase["AggregationRule.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = AggregationRule.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for AggregationRule."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["AggregationRule", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use AggregationRule.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    cluster_role_selectors: Annotated[
        Optional[List[apimachinery.LabelSelector]], Field(alias="clusterRoleSelectors")
    ] = None
    """
    ClusterRoleSelectors holds a list of selectors which will be used to find ClusterRoles and create the rules. If any of the selectors match, then the ClusterRole's permissions will be added
    """


class ClusterRole(Resource):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["ClusterRole"]:
            return ClusterRole

        def build(self) -> "ClusterRole":
            return ClusterRole(**self._attrs)

        @overload
        def aggregation_rule(
            self, value_or_callback: Optional[AggregationRule], /
        ) -> "ClusterRole.Builder": ...

        @overload
        def aggregation_rule(
            self,
            value_or_callback: Callable[
                [AggregationRule.Builder], AggregationRule.Builder | AggregationRule
            ],
            /,
        ) -> "ClusterRole.Builder": ...

        @overload
        def aggregation_rule(
            self, value_or_callback: Never = ...
        ) -> "AggregationRule.BuilderContext": ...

        def aggregation_rule(self, value_or_callback=None, /):
            """
            AggregationRule is an optional field that describes how to build the Rules for this ClusterRole. If AggregationRule is set, then the Rules are controller managed and direct changes to Rules will be stomped by the controller.
            """
            if self._in_context and value_or_callback is None:
                context = AggregationRule.BuilderContext()
                context._parent_builder = self
                context._field_name = "aggregation_rule"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(AggregationRule.builder())
                if isinstance(output, AggregationRule.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("aggregation_rule", value)

        def api_version(self, value: Optional[Literal["rbac.authorization.k8s.io/v1"]], /) -> Self:
            """
            APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
            """
            return self._set("api_version", value)

        def kind(self, value: Optional[Literal["ClusterRole"]], /) -> Self:
            """
            Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
            """
            return self._set("kind", value)

        @overload
        def metadata(
            self, value_or_callback: Optional[apimachinery.ObjectMeta], /
        ) -> "ClusterRole.Builder": ...

        @overload
        def metadata(
            self,
            value_or_callback: Callable[
                [apimachinery.ObjectMeta.Builder],
                apimachinery.ObjectMeta.Builder | apimachinery.ObjectMeta,
            ],
            /,
        ) -> "ClusterRole.Builder": ...

        @overload
        def metadata(
            self, value_or_callback: Never = ...
        ) -> "apimachinery.ObjectMeta.BuilderContext": ...

        def metadata(self, value_or_callback=None, /):
            """
            Standard object's metadata.
            """
            if self._in_context and value_or_callback is None:
                context = apimachinery.ObjectMeta.BuilderContext()
                context._parent_builder = self
                context._field_name = "metadata"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(apimachinery.ObjectMeta.builder())
                if isinstance(output, apimachinery.ObjectMeta.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("metadata", value)

        @overload
        def rules(self, value_or_callback: List[PolicyRule], /) -> "ClusterRole.Builder": ...

        @overload
        def rules(
            self,
            value_or_callback: Callable[
                [GenericListBuilder[PolicyRule, PolicyRule.Builder]],
                GenericListBuilder[PolicyRule, PolicyRule.Builder] | List[PolicyRule],
            ],
            /,
        ) -> "ClusterRole.Builder": ...

        @overload
        def rules(
            self, value_or_callback: Never = ...
        ) -> ListBuilderContext[PolicyRule.Builder]: ...

        def rules(self, value_or_callback=None, /):
            """
            Rules holds all the PolicyRules for this ClusterRole
            """
            if self._in_context and value_or_callback is None:
                context = ListBuilderContext[PolicyRule.Builder]()
                context._parent_builder = self
                context._field_name = "rules"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(PolicyRule.list_builder())
                if isinstance(output, GenericListBuilder):
                    value = output.build()
                else:
                    value = output
            return self._set("rules", value)

    class BuilderContext(BuilderContextBase["ClusterRole.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = ClusterRole.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for ClusterRole."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["ClusterRole", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use ClusterRole.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    aggregation_rule: Annotated[Optional[AggregationRule], Field(alias="aggregationRule")] = None
    """
    AggregationRule is an optional field that describes how to build the Rules for this ClusterRole. If AggregationRule is set, then the Rules are controller managed and direct changes to Rules will be stomped by the controller.
    """
    api_version: Annotated[
        Optional[Literal["rbac.authorization.k8s.io/v1"]], Field(alias="apiVersion")
    ] = "rbac.authorization.k8s.io/v1"
    """
    APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
    """
    kind: Optional[Literal["ClusterRole"]] = "ClusterRole"
    """
    Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
    """
    metadata: Optional[apimachinery.ObjectMeta] = None
    """
    Standard object's metadata.
    """
    rules: Optional[List[PolicyRule]] = None
    """
    Rules holds all the PolicyRules for this ClusterRole
    """


class ClusterRoleBinding(Resource):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["ClusterRoleBinding"]:
            return ClusterRoleBinding

        def build(self) -> "ClusterRoleBinding":
            return ClusterRoleBinding(**self._attrs)

        def api_version(self, value: Optional[Literal["rbac.authorization.k8s.io/v1"]], /) -> Self:
            """
            APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
            """
            return self._set("api_version", value)

        def kind(self, value: Optional[Literal["ClusterRoleBinding"]], /) -> Self:
            """
            Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
            """
            return self._set("kind", value)

        @overload
        def metadata(
            self, value_or_callback: Optional[apimachinery.ObjectMeta], /
        ) -> "ClusterRoleBinding.Builder": ...

        @overload
        def metadata(
            self,
            value_or_callback: Callable[
                [apimachinery.ObjectMeta.Builder],
                apimachinery.ObjectMeta.Builder | apimachinery.ObjectMeta,
            ],
            /,
        ) -> "ClusterRoleBinding.Builder": ...

        @overload
        def metadata(
            self, value_or_callback: Never = ...
        ) -> "apimachinery.ObjectMeta.BuilderContext": ...

        def metadata(self, value_or_callback=None, /):
            """
            Standard object's metadata.
            """
            if self._in_context and value_or_callback is None:
                context = apimachinery.ObjectMeta.BuilderContext()
                context._parent_builder = self
                context._field_name = "metadata"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(apimachinery.ObjectMeta.builder())
                if isinstance(output, apimachinery.ObjectMeta.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("metadata", value)

        @overload
        def role_ref(self, value_or_callback: RoleRef, /) -> "ClusterRoleBinding.Builder": ...

        @overload
        def role_ref(
            self,
            value_or_callback: Callable[[RoleRef.Builder], RoleRef.Builder | RoleRef],
            /,
        ) -> "ClusterRoleBinding.Builder": ...

        @overload
        def role_ref(self, value_or_callback: Never = ...) -> "RoleRef.BuilderContext": ...

        def role_ref(self, value_or_callback=None, /):
            """
            RoleRef can only reference a ClusterRole in the global namespace. If the RoleRef cannot be resolved, the Authorizer must return an error. This field is immutable.
            """
            if self._in_context and value_or_callback is None:
                context = RoleRef.BuilderContext()
                context._parent_builder = self
                context._field_name = "role_ref"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(RoleRef.builder())
                if isinstance(output, RoleRef.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("role_ref", value)

        @overload
        def subjects(self, value_or_callback: List[Subject], /) -> "ClusterRoleBinding.Builder": ...

        @overload
        def subjects(
            self,
            value_or_callback: Callable[
                [GenericListBuilder[Subject, Subject.Builder]],
                GenericListBuilder[Subject, Subject.Builder] | List[Subject],
            ],
            /,
        ) -> "ClusterRoleBinding.Builder": ...

        @overload
        def subjects(
            self, value_or_callback: Never = ...
        ) -> ListBuilderContext[Subject.Builder]: ...

        def subjects(self, value_or_callback=None, /):
            """
            Subjects holds references to the objects the role applies to.
            """
            if self._in_context and value_or_callback is None:
                context = ListBuilderContext[Subject.Builder]()
                context._parent_builder = self
                context._field_name = "subjects"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(Subject.list_builder())
                if isinstance(output, GenericListBuilder):
                    value = output.build()
                else:
                    value = output
            return self._set("subjects", value)

    class BuilderContext(BuilderContextBase["ClusterRoleBinding.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = ClusterRoleBinding.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for ClusterRoleBinding."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["ClusterRoleBinding", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use ClusterRoleBinding.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    api_version: Annotated[
        Optional[Literal["rbac.authorization.k8s.io/v1"]], Field(alias="apiVersion")
    ] = "rbac.authorization.k8s.io/v1"
    """
    APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
    """
    kind: Optional[Literal["ClusterRoleBinding"]] = "ClusterRoleBinding"
    """
    Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
    """
    metadata: Optional[apimachinery.ObjectMeta] = None
    """
    Standard object's metadata.
    """
    role_ref: Annotated[RoleRef, Field(alias="roleRef")]
    """
    RoleRef can only reference a ClusterRole in the global namespace. If the RoleRef cannot be resolved, the Authorizer must return an error. This field is immutable.
    """
    subjects: Optional[List[Subject]] = None
    """
    Subjects holds references to the objects the role applies to.
    """


ClusterRoleBindingList = ResourceList["ClusterRoleBinding"]


ClusterRoleList = ResourceList["ClusterRole"]


class Role(Resource):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["Role"]:
            return Role

        def build(self) -> "Role":
            return Role(**self._attrs)

        def api_version(self, value: Optional[Literal["rbac.authorization.k8s.io/v1"]], /) -> Self:
            """
            APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
            """
            return self._set("api_version", value)

        def kind(self, value: Optional[Literal["Role"]], /) -> Self:
            """
            Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
            """
            return self._set("kind", value)

        @overload
        def metadata(
            self, value_or_callback: Optional[apimachinery.ObjectMeta], /
        ) -> "Role.Builder": ...

        @overload
        def metadata(
            self,
            value_or_callback: Callable[
                [apimachinery.ObjectMeta.Builder],
                apimachinery.ObjectMeta.Builder | apimachinery.ObjectMeta,
            ],
            /,
        ) -> "Role.Builder": ...

        @overload
        def metadata(
            self, value_or_callback: Never = ...
        ) -> "apimachinery.ObjectMeta.BuilderContext": ...

        def metadata(self, value_or_callback=None, /):
            """
            Standard object's metadata.
            """
            if self._in_context and value_or_callback is None:
                context = apimachinery.ObjectMeta.BuilderContext()
                context._parent_builder = self
                context._field_name = "metadata"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(apimachinery.ObjectMeta.builder())
                if isinstance(output, apimachinery.ObjectMeta.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("metadata", value)

        @overload
        def rules(self, value_or_callback: List[PolicyRule], /) -> "Role.Builder": ...

        @overload
        def rules(
            self,
            value_or_callback: Callable[
                [GenericListBuilder[PolicyRule, PolicyRule.Builder]],
                GenericListBuilder[PolicyRule, PolicyRule.Builder] | List[PolicyRule],
            ],
            /,
        ) -> "Role.Builder": ...

        @overload
        def rules(
            self, value_or_callback: Never = ...
        ) -> ListBuilderContext[PolicyRule.Builder]: ...

        def rules(self, value_or_callback=None, /):
            """
            Rules holds all the PolicyRules for this Role
            """
            if self._in_context and value_or_callback is None:
                context = ListBuilderContext[PolicyRule.Builder]()
                context._parent_builder = self
                context._field_name = "rules"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(PolicyRule.list_builder())
                if isinstance(output, GenericListBuilder):
                    value = output.build()
                else:
                    value = output
            return self._set("rules", value)

    class BuilderContext(BuilderContextBase["Role.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = Role.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for Role."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["Role", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use Role.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    api_version: Annotated[
        Optional[Literal["rbac.authorization.k8s.io/v1"]], Field(alias="apiVersion")
    ] = "rbac.authorization.k8s.io/v1"
    """
    APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
    """
    kind: Optional[Literal["Role"]] = "Role"
    """
    Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
    """
    metadata: Optional[apimachinery.ObjectMeta] = None
    """
    Standard object's metadata.
    """
    rules: Optional[List[PolicyRule]] = None
    """
    Rules holds all the PolicyRules for this Role
    """


class RoleBinding(Resource):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["RoleBinding"]:
            return RoleBinding

        def build(self) -> "RoleBinding":
            return RoleBinding(**self._attrs)

        def api_version(self, value: Optional[Literal["rbac.authorization.k8s.io/v1"]], /) -> Self:
            """
            APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
            """
            return self._set("api_version", value)

        def kind(self, value: Optional[Literal["RoleBinding"]], /) -> Self:
            """
            Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
            """
            return self._set("kind", value)

        @overload
        def metadata(
            self, value_or_callback: Optional[apimachinery.ObjectMeta], /
        ) -> "RoleBinding.Builder": ...

        @overload
        def metadata(
            self,
            value_or_callback: Callable[
                [apimachinery.ObjectMeta.Builder],
                apimachinery.ObjectMeta.Builder | apimachinery.ObjectMeta,
            ],
            /,
        ) -> "RoleBinding.Builder": ...

        @overload
        def metadata(
            self, value_or_callback: Never = ...
        ) -> "apimachinery.ObjectMeta.BuilderContext": ...

        def metadata(self, value_or_callback=None, /):
            """
            Standard object's metadata.
            """
            if self._in_context and value_or_callback is None:
                context = apimachinery.ObjectMeta.BuilderContext()
                context._parent_builder = self
                context._field_name = "metadata"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(apimachinery.ObjectMeta.builder())
                if isinstance(output, apimachinery.ObjectMeta.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("metadata", value)

        @overload
        def role_ref(self, value_or_callback: RoleRef, /) -> "RoleBinding.Builder": ...

        @overload
        def role_ref(
            self,
            value_or_callback: Callable[[RoleRef.Builder], RoleRef.Builder | RoleRef],
            /,
        ) -> "RoleBinding.Builder": ...

        @overload
        def role_ref(self, value_or_callback: Never = ...) -> "RoleRef.BuilderContext": ...

        def role_ref(self, value_or_callback=None, /):
            """
            RoleRef can reference a Role in the current namespace or a ClusterRole in the global namespace. If the RoleRef cannot be resolved, the Authorizer must return an error. This field is immutable.
            """
            if self._in_context and value_or_callback is None:
                context = RoleRef.BuilderContext()
                context._parent_builder = self
                context._field_name = "role_ref"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(RoleRef.builder())
                if isinstance(output, RoleRef.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("role_ref", value)

        @overload
        def subjects(self, value_or_callback: List[Subject], /) -> "RoleBinding.Builder": ...

        @overload
        def subjects(
            self,
            value_or_callback: Callable[
                [GenericListBuilder[Subject, Subject.Builder]],
                GenericListBuilder[Subject, Subject.Builder] | List[Subject],
            ],
            /,
        ) -> "RoleBinding.Builder": ...

        @overload
        def subjects(
            self, value_or_callback: Never = ...
        ) -> ListBuilderContext[Subject.Builder]: ...

        def subjects(self, value_or_callback=None, /):
            """
            Subjects holds references to the objects the role applies to.
            """
            if self._in_context and value_or_callback is None:
                context = ListBuilderContext[Subject.Builder]()
                context._parent_builder = self
                context._field_name = "subjects"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(Subject.list_builder())
                if isinstance(output, GenericListBuilder):
                    value = output.build()
                else:
                    value = output
            return self._set("subjects", value)

    class BuilderContext(BuilderContextBase["RoleBinding.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = RoleBinding.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for RoleBinding."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["RoleBinding", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use RoleBinding.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    api_version: Annotated[
        Optional[Literal["rbac.authorization.k8s.io/v1"]], Field(alias="apiVersion")
    ] = "rbac.authorization.k8s.io/v1"
    """
    APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
    """
    kind: Optional[Literal["RoleBinding"]] = "RoleBinding"
    """
    Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
    """
    metadata: Optional[apimachinery.ObjectMeta] = None
    """
    Standard object's metadata.
    """
    role_ref: Annotated[RoleRef, Field(alias="roleRef")]
    """
    RoleRef can reference a Role in the current namespace or a ClusterRole in the global namespace. If the RoleRef cannot be resolved, the Authorizer must return an error. This field is immutable.
    """
    subjects: Optional[List[Subject]] = None
    """
    Subjects holds references to the objects the role applies to.
    """


RoleBindingList = ResourceList["RoleBinding"]


RoleList = ResourceList["Role"]
